use chrono::Utc;
use sea_orm_migration::{prelude::*, schema::*};

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .create_table(
                Table::create()
                    .table(Benutzer::Table)
                    .if_not_exists()
                    .col(pk_uuid(Benutzer::Id))
                    .col(string(Benutzer::EMail).not_null())
                    .col(string(Benutzer::Kennung))
                    .col(string_null(Benutzer::PasswordHash))
                    .col(string(Benutzer::Nachname))
                    .col(string(Benutzer::Vorname))
                    .col(
                        timestamp_with_time_zone(Benutzer::ErstelltAm)
                            .not_null()
                            .default(Utc::now()),
                    )
                    .col(
                        timestamp_with_time_zone(Benutzer::GeaendertAm)
                            .not_null()
                            .default(Utc::now()),
                    )
                    .col(boolean(Benutzer::IstAktiv).default(true))
                    .to_owned(),
            )
            .await?;

        manager
            .create_index(
                Index::create()
                    .unique()
                    .name("idx_benutzer_email")
                    .table(Benutzer::Table)
                    .col(Benutzer::EMail)
                    .to_owned(),
            )
            .await?;

        manager
            .create_index(
                Index::create()
                    .unique()
                    .name("idx_benutzer_kennung")
                    .table(Benutzer::Table)
                    .col(Benutzer::Kennung)
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(Table::drop().table(Benutzer::Table).to_owned())
            .await
    }
}

#[derive(DeriveIden)]
pub enum Benutzer {
    Table,
    Id,
    EMail,
    Kennung,
    PasswordHash,
    Nachname,
    Vorname,
    ErstelltAm,
    GeaendertAm,
    IstAktiv,
}
